;-------------------------------------------------------------------------
;
;	Sublimation
;	by Dave Seidel (dave at mysterybear dot net)
;	see http://mysterybear.net
;
;	Sublimation (Realtime Version) by Art Hunkins (abhunkin at uncg dot edu)
;	see http://www.arthunkins.com
;
;   Copyright 2006, Dave Seidel. Some rights reserved.
;   This work is licensed under a Creative Commons "Attribution" License.
;   http://creativecommons.org/licenses/by/2.5/
;
;	Tested with Csound 5.00RC2 on Windows XP.
;	Realtime Versions tested with Csound 5.00 on Windows XP, ME.
;
;-------------------------------------------------------------------------

<CsoundSynthesizer>
<CsOptions>

-odac0 -m0d -M0 -b128 -B8192 --expression-opt

</CsOptions>
<CsInstruments>

;-------------------------------------------------------------------------
; globals
;-------------------------------------------------------------------------

#define CHAN	#1# ; CHAN = 0 means multichannel, controller 7 input
#define CTRL1	#20#
#define SRATE	#48000# ; SRATE choices: 22050, 32000, 44100, 48000 (the higher the better)

	ctrlinit	$CHAN, 7, 0, 8, 0, 10, 0, 11, 0
	ctrlinit	1, 7, 0
	ctrlinit	2, 7, 0
	ctrlinit	3, 7, 0
	ctrlinit	4, 7, 0
	ctrlinit	5, 7, 0
	ctrlinit	6, 7, 0
	ctrlinit	7, 7, 0
	ctrlinit	8, 7, 0

sr	=	$SRATE
ksmps 	=	100
nchnls	=	2

;-------------------------------------------------------------------------
; global reverb sends
;-------------------------------------------------------------------------

gaL	init	0
gaR	init	0

gktrig	init	0
giamp	=	2000/9

;-------------------------------------------------------------------------
; function table for waveform
;-------------------------------------------------------------------------

; sine
giFn1 	ftgen	1, 0, 262144, 10, 1

;---------------------------------------------------------------------------
;	global reverb instrument
;---------------------------------------------------------------------------

	instr 1

ic8	init	.2
kc7	ctrl7	($CHAN == 0? 7: $CHAN), ($CHAN == 0? 7: $CTRL1 + 6), .95, .75
kc8	ctrl7	($CHAN == 0? 8: $CHAN), ($CHAN == 0? 7: $CTRL1 + 7), .2, .6
	if kc8 == ic8 goto skip
	reinit	init
init:
ic8	=	i(kc8)
skip:
aL, aR	reverbsc	gaL, gaR, kc7, 15360, sr/1.5, ic8, 0
	rireturn
	outs	gaL+aL, gaR+aR

gaL	=	0
gaR	=	0

	endin

	instr 2

gkc1	ctrl7	($CHAN == 0? 1: $CHAN), ($CHAN == 0? 7: $CTRL1), -.01, 1
gkc2	ctrl7	($CHAN == 0? 2: $CHAN), ($CHAN == 0? 7: $CTRL1 + 1), -.01, 1
gkc3	ctrl7	($CHAN == 0? 3: $CHAN), ($CHAN == 0? 7: $CTRL1 + 2), -.01, 1
gkc4	ctrl7	($CHAN == 0? 4: $CHAN), ($CHAN == 0? 7: $CTRL1 + 3), -.01, 1
gkc5	ctrl7	($CHAN == 0? 5: $CHAN), ($CHAN == 0? 7: $CTRL1 + 4), -.01, 1
gkc6	ctrl7	($CHAN == 0? 6: $CHAN), ($CHAN == 0? 7: $CTRL1 + 5), -.01, 1
gkc1	port	gkc1, .01
gkc2	port	gkc2, .01
gkc3	port	gkc3, .01
gkc4	port	gkc4, .01
gkc5	port	gkc5, .01
gkc6	port	gkc6, .01
kc1	trigger	gkc1, 0, 0
	schedkwhen kc1, 0, 0, 3.1, 0, -1, 120, 3, 2, .9
	schedkwhen kc1, 0, 0, 3.2, 0, -1, 180, 7, 6, .7
	schedkwhen kc1, 0, 0, 3.3, 0, -1, 210, 8, 7, .5
	schedkwhen kc1, 0, 0, 3.4, 0, -1, 240, 9, 8, .3
	schedkwhen kc1, 0, 0, 3.5, 0, -1, 270, 4, 3, .1
kc2	trigger	gkc2, 0, 0
	schedkwhen kc2, 0, 0, 4.1, 0, -1, 4, .96
	schedkwhen kc2, 0, 0, 4.2, 0, -1, 6, .8
	schedkwhen kc2, 0, 0, 4.3, 0, -1, 7, .64
	schedkwhen kc2, 0, 0, 4.4, 0, -1, 8, .48
	schedkwhen kc2, 0, 0, 4.5, 0, -1, 9, .32
	schedkwhen kc2, 0, 0, 4.6, 0, -1, 12, .16
kc3	trigger	gkc3, 0, 0
	schedkwhen kc3, 0, 0, 5.1, 0, -1, 81, 1
	schedkwhen kc3, 0, 0, 5.2, 0, -1, 84, .875
	schedkwhen kc3, 0, 0, 5.3, 0, -1, 108, .75
	schedkwhen kc3, 0, 0, 5.4, 0, -1, 112, .625
	schedkwhen kc3, 0, 0, 5.5, 0, -1, 144, .5
	schedkwhen kc3, 0, 0, 5.6, 0, -1, 162, .375
	schedkwhen kc3, 0, 0, 5.7, 0, -1, 192, .25
	schedkwhen kc3, 0, 0, 5.8, 0, -1, 216, .125
kc4	trigger	gkc4, 0, 0
	schedkwhen kc4, 0, 0, 6.1, 0, -1, 512, .990
	schedkwhen kc4, 0, 0, 6.2, 0, -1, 567, .9186
	schedkwhen kc4, 0, 0, 6.3, 0, -1, 588, .8472
	schedkwhen kc4, 0, 0, 6.4, 0, -1, 756, .7758
	schedkwhen kc4, 0, 0, 6.5, 0, -1, 784, .7044
	schedkwhen kc4, 0, 0, 6.6, 0, -1, 784, .633
	schedkwhen kc4, 0, 0, 6.7, 0, -1, 896, .5616
kc4a	trigger	gkc4, .01, 0
	schedkwhen kc4a, 0, 0, 6.8, 0, -1, 1008, .4902
	schedkwhen kc4a, 0, 0, 6.9, 0, -1, 1024, .4188
	schedkwhen kc4a, 0, 0, 6.10, 0, -1, 1134, .3474
	schedkwhen kc4a, 0, 0, 6.11, 0, -1, 1152, .276
	schedkwhen kc4a, 0, 0, 6.12, 0, -1, 1344, .2046
	schedkwhen kc4a, 0, 0, 6.13, 0, -1, 1512, .1332
	schedkwhen kc4a, 0, 0, 6.14, 0, -1, 1546, .0618
kc5	trigger	gkc5, 0, 0
	schedkwhen kc5, 0, 0, 7.1, 0, -1, 151.875, 28, 27, .8571
	schedkwhen kc5, 0, 0, 7.2, 0, -1, 157.5, 9, 7, .7142
	schedkwhen kc5, 0, 0, 7.3, 0, -1, 202.5, 28, 27, .5713
	schedkwhen kc5, 0, 0, 7.4, 0, -1, 210, 9, 7, .4284
	schedkwhen kc5, 0, 0, 7.5, 0, -1, 270, 9, 8, .2855
	schedkwhen kc5, 0, 0, 7.6, 0, -1, 303.75, 32, 27, .1426
	schedkwhen kc5, 0, 0, 7.7, 0, -1, 360, 9, 8, 0 
kc6	trigger	gkc6, 0, 0
	schedkwhen kc6, 0, 0, 8.1, 0, -1, 4, 1
	schedkwhen kc6, 0, 0, 8.2, 0, -1, 6, .8
	schedkwhen kc6, 0, 0, 8.3, 0, -1, 7, .6
	schedkwhen kc6, 0, 0, 8.4, 0, -1, 8, .4
	schedkwhen kc6, 0, 0, 8.5, 0, -1, 9, .2
	schedkwhen kc6, 0, 0, 8.6, 0, -1, 12, 0
kon	trigger	gkc1 + gkc2 + gkc3, 2.99, 0
	if kon == 0 kgoto skip
gktrig	=	1
	kgoto skip2
skip:
koff	trigger	gkc1 + gkc2 + gkc3, 0, 1
	if koff == 0 kgoto skip2
gktrig	=	0
skip2:
gkc2	=	(gktrig == 1? gkc1: gkc2)
gkc3	=	(gktrig == 1? gkc1: gkc3)

	endin

;---------------------------------------------------------------------------------------
; combination tone chord organ: given a note and a ratio, produces the chord formed
; by that interval plus the set of notes equivalent to the combination tones produced
; by that interval
;---------------------------------------------------------------------------------------

	instr 3

ipch1	=	p4
ipch2	=	p4*(p5/p6)
idiff	=	ipch2-ipch1
idiff2	=	(2*ipch2)-ipch1
idiff3	=	(3*ipch2)-(2*ipch1)
iper	=	ipch1/p6
isum	=	ipch1+ipch2
isum2	=	(2*ipch1)+ipch2

kamp	=	giamp*gkc1
asig2	oscili	kamp, ipch2, 1
adiff	oscili	kamp, idiff, 1
adiff2	oscili	kamp, idiff2, 1
adiff3	oscili	kamp, idiff3, 1
aper	oscili	kamp, iper, 1
asum	oscili	kamp, isum, 1
asum2	oscili	kamp, isum2, 1

aout	=	asig2+adiff+adiff2+adiff3+aper+asum+asum2	

gaL	=	gaL+(aout*p7)
gaR	=	gaR+(aout*(1-p7))
koff	trigger	gkc1, 0, 1
	if koff = 0 goto end
	turnoff

end:	endin

	instr 4

ipch1	=	30
ipch2	=	30*p4
idiff	=	ipch2-ipch1
idiff2	=	(2*ipch2)-ipch1
idiff3	=	(3*ipch2)-(2*ipch1)
isum	=	ipch1+ipch2
isum2	=	(2*ipch1)+ipch2

kamp	=	giamp*gkc2
asig2	oscili	kamp, ipch2, 1
adiff	oscili	kamp, idiff, 1
adiff2	oscili	kamp, idiff2, 1
adiff3	oscili	kamp, idiff3, 1
aper	oscili	kamp, ipch1, 1
asum	oscili	kamp, isum, 1
asum2	oscili	kamp, isum2, 1

aout	=	asig2+adiff+adiff2+adiff3+aper+asum+asum2

gaL	=	gaL+(aout*p5)
gaR	=	gaR+(aout*(1-p5))
koff	trigger	gkc2, 0, 1
	if koff = 0 goto end
	turnoff

end:	endin

	instr 5

ipch1	=	1.875
ipch2	=	1.875*p4
idiff	=	ipch2-ipch1
idiff2	=	(2*ipch2)-ipch1
idiff3	=	(3*ipch2)-(2*ipch1)
isum	=	ipch1+ipch2
isum2	=	(2*ipch1)+ipch2

kamp	=	giamp*gkc3
asig2	oscili	kamp, ipch2, 1
adiff	oscili	kamp, idiff, 1
adiff2	oscili	kamp, idiff2, 1
adiff3	oscili	kamp, idiff3, 1
asum	oscili	kamp, isum, 1
asum2	oscili	kamp, isum2, 1

aout	=	asig2+adiff+adiff2+adiff3+asum+asum2

gaL	=	gaL+(aout*p5)
gaR	=	gaR+(aout*(1-p5))
koff	trigger	gkc3, 0, 1
	if koff = 0 goto end
	turnoff

end:	endin

	instr 6

ipch1	=	.46875
ipch2	=	.46875*p4
idiff	=	ipch2-ipch1
idiff2	=	(2*ipch2)-ipch1
idiff3	=	(3*ipch2)-(2*ipch1)
isum	=	ipch1+ipch2
isum2	=	(2*ipch1)+ipch2

kamp	=	giamp*gkc4
asig2	oscili	kamp, ipch2, 1
adiff	oscili	kamp, idiff, 1
adiff2	oscili	kamp, idiff2, 1
adiff3	oscili	kamp, idiff3, 1
asum	oscili	kamp, isum, 1
asum2	oscili	kamp, isum2, 1

aout	=	asig2+adiff+adiff2+adiff3+asum+asum2	

gaL	=	gaL+(aout*p5)
gaR	=	gaR+(aout*(1-p5))
koff	trigger	gkc4, 0, 1
	if koff = 0 goto end
	turnoff

end:	endin

	instr 7

ipch1	=	p4
ipch2	=	p4*(p5/p6)
idiff	=	ipch2-ipch1
idiff2	=	(2*ipch2)-ipch1
idiff3	=	(3*ipch2)-(2*ipch1)
isum	=	ipch1+ipch2
isum2	=	(2*ipch1)+ipch2

kamp	=	giamp*gkc5
asig2	oscili	kamp, ipch2, 1
adiff	oscili	kamp, idiff, 1
adiff2	oscili	kamp, idiff2, 1
adiff3	oscili	kamp, idiff3, 1
asum	oscili	kamp, isum, 1
asum2	oscili	kamp, isum2, 1

aout	=	asig2+adiff+adiff2+adiff3+asum+asum2

gaL	=	gaL+(aout*p7)
gaR	=	gaR+(aout*(1-p7))
koff	trigger	gkc5, 0, 1
	if koff = 0 goto end
	turnoff

end:	endin

	instr 8

ipch1	=	30
ipch2	=	30*p4
idiff	=	ipch2-ipch1
idiff2	=	(2*ipch2)-ipch1
idiff3	=	(3*ipch2)-(2*ipch1)
isum	=	ipch1+ipch2
isum2	=	(2*ipch1)+ipch2

kamp	=	giamp*gkc6
asig2	oscili	kamp, ipch2, 1
adiff	oscili	kamp, idiff, 1
adiff2	oscili	kamp, idiff2, 1
adiff3	oscili	kamp, idiff3, 1
aper	oscili	kamp, ipch1, 1
asum	oscili	kamp, isum, 1
asum2	oscili	kamp, isum2, 1

aout	=	asig2+adiff+adiff2+adiff3+aper+asum+asum2

gaL	=	gaL+(aout*p5)
gaR	=	gaR+(aout*(1-p5))
koff	trigger	gkc6, 0, 1
	if koff = 0 goto end
	turnoff

end:	endin

</CsInstruments>
<CsScore>

;---------------------------------------------------------------------------
; score
;---------------------------------------------------------------------------

i 1 0 3600
i 2 0 3600

e

</CsScore>
</CsoundSynthesizer>